@@ -1,6 +1,21 @@
-/*
- * Copyright (C) 2003 Joachim Eibl <joachim.eibl@gmx.de>
- */
+/***************************************************************************
+ * Copyright (C) 2003-2006 Joachim Eibl <joachim.eibl at gmx.de>           *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
+ ***************************************************************************/
 
 #include "kdiff3_part.h"
 
@@ -161,8 +176,8 @@ bool KDiff3Part::openFile()
 
       m_widget->slotFileOpen2( fileName1, tempFileName, "", "",
                                "", version2.isEmpty() ? fileName2 : "REV:"+version2+":"+fileName2, "", 0 ); // alias names
-   std::cerr << "KDiff3: f1:" << fileName1.latin1() <<"<->"<<tempFileName.latin1()<< std::endl;
-//      FileAccess::removeFile( tempFileName );
+//    std::cerr << "KDiff3: f1:" << fileName1.latin1() <<"<->"<<tempFileName.latin1()<< std::endl;
+      FileAccess::removeTempFile( tempFileName );
    }
    else if ( version2.isEmpty() && f2.exists() )
    {
@@ -176,7 +191,8 @@ bool KDiff3Part::openFile()
 
       m_widget->slotFileOpen2( tempFileName, fileName2, "", "",
                                version1.isEmpty() ? fileName1 : "REV:"+version1+":"+fileName1, "", "", 0 ); // alias name
-   std::cerr << "KDiff3: f2:" << fileName2.latin1() <<"<->"<<tempFileName.latin1()<< std::endl;
+//    std::cerr << "KDiff3: f2:" << fileName2.latin1() <<"<->"<<tempFileName.latin1()<< std::endl;
+      FileAccess::removeTempFile( tempFileName );
    }
    else if ( !version1.isEmpty() && !version2.isEmpty() )
    {
@@ -198,9 +214,9 @@ bool KDiff3Part::openFile()
          "", 0
       );
 
-   std::cerr << "KDiff3: f1/2:" << tempFileName1.latin1() <<"<->"<<tempFileName2.latin1()<< std::endl;
-//      FileAccess::removeFile( tempFileName1 );
- //     FileAccess::removeFile( tempFileName2 );
+//    std::cerr << "KDiff3: f1/2:" << tempFileName1.latin1() <<"<->"<<tempFileName2.latin1()<< std::endl;
+      FileAccess::removeTempFile( tempFileName1 );
+      FileAccess::removeTempFile( tempFileName2 );
       return true;
    }
    else
@@ -273,7 +289,7 @@ KInstance* KDiff3PartFactory::instance()
     if( !s_instance )
     {
         s_about = new KAboutData("kdiff3part", I18N_NOOP("KDiff3Part"), VERSION);
-        s_about->addAuthor("Joachim Eibl", 0, "joachim.eibl@gmx.de");
+        s_about->addAuthor("Joachim Eibl", 0, "joachim.eibl at gmx.de");
         s_instance = new KInstance(s_about);
     }
     return s_instance;
